@@ -239,7 +239,7 @@ __all__ = ["compile", "escape", "findall", "finditer", "fullmatch", "match",
   "U", "UNICODE", "V0", "VERSION0", "V1", "VERSION1", "X", "VERBOSE", "W",
   "WORD", "error", "Regex"]
 
-__version__ = "2.4.81"
+__version__ = "2.4.88"
 
 # --------------------------------------------------------------------
 # Public interface.
@@ -355,50 +355,38 @@ def template(pattern, flags=0):
 
 def escape(pattern, special_only=False):
     "Escape all non-alphanumeric characters or special characters in pattern."
-    if isinstance(pattern, str):
-        s = []
-        if special_only:
-            for c in pattern:
-                if c in _METACHARS:
-                    s.append("\\")
-                    s.append(c)
-                elif c == "\x00":
-                    s.append("\\000")
-                else:
-                    s.append(c)
-        else:
-            for c in pattern:
-                if c in _ALNUM:
-                    s.append(c)
-                elif c == "\x00":
-                    s.append("\\000")
-                else:
-                    s.append("\\")
-                    s.append(c)
-
-        return "".join(s)
+    # Convert it to Unicode.
+    if isinstance(pattern, bytes):
+        p = pattern.decode("latin-1")
     else:
-        s = []
-        if special_only:
-            for c in pattern:
-                if chr(c) in _METACHARS:
-                    s.extend(b"\\")
-                    s.append(c)
-                elif c == 0:
-                    s.extend(b"\\000")
-                else:
-                    s.append(c)
-        else:
-            for c in pattern:
-                if chr(c) in _ALNUM:
-                    s.append(c)
-                elif c == 0:
-                    s.extend(b"\\000")
-                else:
-                    s.extend(b"\\")
-                    s.append(c)
+        p = pattern
 
-        return bytes(s)
+    s = []
+    if special_only:
+        for c in p:
+            if c in _METACHARS:
+                s.append("\\")
+                s.append(c)
+            elif c == "\x00":
+                s.append("\\000")
+            else:
+                s.append(c)
+    else:
+        for c in p:
+            if c in _ALNUM:
+                s.append(c)
+            elif c == "\x00":
+                s.append("\\000")
+            else:
+                s.append("\\")
+                s.append(c)
+
+    r = "".join(s)
+    # Convert it back to bytes if necessary.
+    if isinstance(pattern, bytes):
+        r = r.encode("latin-1")
+
+    return r
 
 # --------------------------------------------------------------------
 # Internals.
@@ -491,10 +479,10 @@ def _compile(pattern, flags=0, kwargs={}):
     # Set the default version in the core code in case it has been changed.
     _regex_core.DEFAULT_VERSION = DEFAULT_VERSION
 
-    caught_exception = None
     global_flags = flags
 
     while True:
+        caught_exception = None
         try:
             source = _Source(pattern)
             info = _Info(global_flags, source.char_type, kwargs)
@@ -538,11 +526,8 @@ def _compile(pattern, flags=0, kwargs={}):
     # Remember whether this pattern as an inline locale flag.
     _locale_sensitive[locale_key] = info.inline_locale
 
-    # Should we print the parsed pattern?
-    if flags & DEBUG:
-        parsed.dump(indent=0, reverse=reverse)
-
     # Fix the group references.
+    caught_exception = None
     try:
         parsed.fix_groups(pattern, reverse, False)
     except error as e:
@@ -552,8 +537,12 @@ def _compile(pattern, flags=0, kwargs={}):
         raise error(caught_exception.msg, caught_exception.pattern,
           caught_exception.pos)
 
+    # Should we print the parsed pattern?
+    if flags & DEBUG:
+        parsed.dump(indent=0, reverse=reverse)
+
     # Optimise the parsed pattern.
-    parsed = parsed.optimise(info)
+    parsed = parsed.optimise(info, reverse)
     parsed = parsed.pack_characters(info)
 
     # Get the required string.
